;
; Test program
;
; Created: 2008-07-15
; Updated: 2008-09-28 
;
; Goal: 
;  * Try to use name table extracted from emulator
;  - Make a CFG for bank switching capabilities (MMC3 + CHR-RAM)
;  * Convert util.asm to module and import symbols
;  * Fix palette error
;  * Put dummy spr data in dummy file
;
; Comments: 
;   - some code used from nes Wiki and Blarg's tutorial
;-----------------------------------------------------------------------------
; CHR ROM data
.segment "CHARS"
	.incbin "./data/chr/bc.chr"
	
; Vectors table
.segment "VECTORS"
	.word 0,0,0, vector_nmi, vector_reset, vector_irq

; PRG ROM data
.segment "CODE"

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-
; Export some zero page variables that 
; can be used as parameters in some 
; external modules
;
.exportzp zpJoy1Status
.exportzp zpCopySrcAdr
.exportzp zpCopyDestAdr
.exportzp zpNmiCount

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; Temporary ZP variable, may be refactored
;
.zeropage

zpJoy1Status:          .res 1 ; Status of first joystick
zpNmiCount:            .res 1 ; This reserves 1 byte for nmi counter
zpPpuCtrlFlags:        .res 1 ; backup state of PPU_CTRL flags
zpPpuMaskFlags:        .res 1 ; backup state of PPU_MASK flags

; Temp variables
zpCopySrcAdr:          .res 2 ; Source address when copying data
zpCopyDestAdr:         .res 2 ; Destination address when copying data
zpRefreshPpuCtrlFlags: .res 1 ; When set, must refresh PPU_CTRL (tentative)

.code
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-
; Include some useful symbolic constants,
; macros and headers files
;
.include "./src/inc/NES.inc"
.include "./src/common/utils.h"
.include "./src/initDummyData.h"


;->BEGIN----------------------------------------------------------------------
; The reset and entry point of the application
;
vector_reset:
	; This must be done BEFORE calling any routines, since
	; it sets the stack pointer
	ldx #$FF
	txs	
	
	; Call routine to initialize NES
	jsr subInitNes
	
	; Now load dummy test data
	jsr subInitAllDummyData
	
	; Then set the display parameters
	jsr subInitDefaultDisplayParam
	
;-<END------------------------------------------------------------------------	


;->BEGIN----------------------------------------------------------------------
; Main loop of the game
;
mainLoop:
	
;-=-=-=-=-=--==-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
; NOTE: The following code is experimental and
;       must be refactored
;-=-=-=-=-=--==-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

	; Init flag
	lda #0
	sta zpRefreshPpuCtrlFlags

	; Read controller button statuses into A
	; TODO: After doing the read, should I check the value in 
	;       zpJoy1Status or A is enough?
	jsr subReadJoy1
	
	; Check each direction and take action if pressed
	; Move top bit into carry and shift other bits left
	asl a
	bcc @not_right  ; if carry is clear, it was not right
	
	inx             ; move right
	
	pha             ; Save A 
	txa             ; Prepare to test x with 0
	cmp #0          ; The scroll wrapped?
	
	bne @not_wrapped_right ; No, continue

    	; Must refresh PPU_CTRL, set flag
    	lda #%00000001
    	sta zpRefreshPpuCtrlFlags
        
	lda zpPpuCtrlFlags ; Load last state
	eor #%00000001     ; Reverse value of nametable flag
	sta zpPpuCtrlFlags ; now save it back
	
@not_wrapped_right:	
	pla             ; In both cases, we have to restore A

@not_right:
	
	asl a
	bcc @not_left   ; clear? t was not left
	dex
	
	pha             ; Save A 
	txa             ; Prepare to test x with $FF
	cmp #$FF        ; The scroll wrapped left?
	
	bne @not_wrapped_left ; No, continue

    	; Must refresh PPU_CTRL, set flag
    	lda #%00000001
    	sta zpRefreshPpuCtrlFlags
        
	lda zpPpuCtrlFlags ; Load last state
	eor #%00000001     ; Reverse value of nametable flag
	sta zpPpuCtrlFlags ; now save it back
	
@not_wrapped_left:	
	pla             ; In both cases, we have to restore A	
	
@not_left:
	asl a
	bcc @not_down   ; clear? It was not down
	iny             ; move down
	
@not_down:
	asl a
	bcc @not_up     ; clear? It was not up
	dey             ; move up
	
@not_up:
	asl a
	bcc @not_start  ; if start, reset values of scroll
	ldx #70
	ldy #0

@not_start:	
	; Wait NMI before possibly updating PPU_CTRL flags and scrolling
	jsr subWaitNmi
	
	; Check if we need to update the PPU because of wrapping
	lda #%00000001
	cmp zpRefreshPpuCtrlFlags
	
	bne @PPUCTRLRefreshNotRequired
	
	; Update PPU CTRL flags
	lda zpPpuCtrlFlags
	sta PPU_CTRL
	
@PPUCTRLRefreshNotRequired:
	
	; Scroll to new position
	jsr subSetScroll

	jmp mainLoop
;-<END------------------------------------------------------------------------


;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; General functions
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


;->BEGIN----------------------------------------------------------------------
; Set the default display parameters
;
.proc subInitDefaultDisplayParam
	;Enable background display
	lda #PPU_MASK_BG_ON + PPU_MASK_BG_NOCLIP + PPU_MASK_SPR_ON
	sta zpPpuMaskFlags  ; Keep backup of state of flags
	sta PPU_MASK 
		
	; init default value for ppu_crtl flags + activate NMI
	lda #PPU_CTRL_NMI + PPU_CTRL_BG_PT1
	sta zpPpuCtrlFlags
	sta PPU_CTRL
	
	; Set scroll to origin (for now begining of land), 70 pixels from left
	ldx #70            
	ldy #0
	
	jsr subSetScroll	

	rts
.endproc
;-<END------------------------------------------------------------------------


;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; Vectors for NMI and IRQ
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


;->BEGIN----------------------------------------------------------------------
; NMI interrupt handler. Called 60 times per second (NTSC)
;
.proc vector_nmi
        ; Increment nmi_count variable
	inc zpNmiCount	
	
	; Update SPR-RAM
	LDA #<OAM
	STA PPU_OAM_ADDR ; set the low byte (00) of the RAM address
	LDA #>OAM
	STA SPR_DMA	
	
	rti
.endproc
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------	
; IRQ interrupt handler (not used in this example)	  
;
.proc vector_irq
    	rti
.endproc    	
;-<END------------------------------------------------------------------------